#~ Cube4.py
#~ Charles Bowman
#~ June 2005

#~ Plot Channel Records from Qub
	# To use: make a .txt file from Qub using Extract
	#~ The ordering of the columns reflects the order in which the A/D channels are polled (I guess).
	#~ So, A/D Ch 0 is column 2 below, A/D Ch 1 is column 3, etc
	#~ Qub hardware configuration settings are listed at the end of this file...
	#~ My input txt file from have the following columns:
		#~ column 	1	time (sec)
		#~ column 	2	membrane potential (mV)
		#~ column	3	current (pA)
		#~ column	4	pressure (mmHg)
		#~ column	5	patch clamp gain (see below for table)
		#~ column 6      Valve 1
		#~ column 7      Valve 2
		#~ column 8      R
		#~ column 9      Capacitance
		#~ column 10    In Phase
	#~ If you do not have patch clamp gain, comment out the relevent section below...
	#~ If your ordering is different, rearrange the relevent sections of the program below
	
	#~ Output: an eps file showing the data. View using ghostview.

#~ Requirements:
	#~ Python: http://www.pyton.org
	#~ PyX: http://pyx.sourceforge.net/
	#~ MikTeX: http://www.miktex.org/
	#~ Ghostscript: http://www.cs.wisc.edu/~ghost/doc/AFPL/get814.htm
	#~ GSview: http://www.cs.wisc.edu/~ghost/gsview/index.htm
	#~ Install MikTeX, then ghostscript, then GSview

from pyx import *
from scipy import *
from pyx.graph.axis.painter import regular
from pyx.deco import barrow, earrow
from math import exp
import sys,re,math
from pyx import prolog

import os.path


# QUB imports
import QUB
from QUBB import *
from Buttons import *
#


#~ prolog.fontdefinition.outputPS = lambda self, file: None

text.set(mode='latex')
#~ text.preamble(r'\usepackage{fancybox}')

ExpInfo = ''

#~ Change this stuff.....
#~ =========================================	
#~ pathToFile = 'D:\\Charley\\AcquisitionInput\\HEK_Cells\\'
#~ pathToFile = 'D:\\Charley\\AcquisitionInput\\CHO_Cells\\Analysis\\'
#~ pathToFile = 'D:\\Charley\\AcquisitionInput\\CHO_Cells\\Raw_CHO\\'
#~ pathToFile = 'D:\\Charley\\AcquisitionInput\\Capacitance\\'
##pathToFile = 'D:\\Charley\\AcquisitionInput\\CHO_Cells\\TrpC\\'
#~ pathToFile = 'D:\\'
##TextFileName = "June132005cx" #name of txt file holding data (w/o .txt extension)

#========================================
##Max_mV = 50  # max membrane potential
##Min_mV = -50 # min membrane potential
##Max_pA = 10  # max membrane current;   assumes patch clamp gain = 100 mV/pA; see below
##Min_pA = -10 # min membrane current;   assumes patch clamp gain = 100 mV/pA; see below
##Max_mmHg = 5 # max pressure
##Min_mmHg  = -100  # min pressure

##mVperfF = 0.0102 # mV per femtoFarad
##Min_Capacitance = -1  # in fF
##Max_Capacitance = 1  # in fF

##Min_InPhase = -.2
##Max_InPhase = .2

##Min_Gain = .1  # min patch clamp gain
##Max_Gain = 1000 # max patch clamp gain
#=======================================
##Min_Valve_1 = -2
##Max_Valve_1 = 11
##Min_Valve_2 = -2
##Max_Valve_2 = 11

##PlotWidth = 18 # (cm)
##PlotHeight = 2 # (cm)
##PlotXStart = 2  # (cm)
##skip = 2.5  # cm distance between plots

#~ ===========================
#~ ========================================
def findScaling(object):
	# Calculate scaling for Ghostview32....use these values to measure  stuff on the plot...
	object.finish()
	#~ Place a bounding box around the current plot.......
	#~ print canvas.canvas.bbox(current),'   points'
	print ''
	xzero,yzero = object.pos(0,0)
	xten,yzero = object.pos(10,0)
	xzero,yten = object.pos(0,10)
	x_diff = xten - xzero
	y_diff = yten - yzero
	#~ print dir(object.__getattribute__)
	#~ print 'Number of cm from 0 to 10 seconds = ',diff,'  cm'
	#~ print 'Number of points from 0 to 10 seconds = ',diff*2.8346
	xscaling = round(1.0/(x_diff*2.8346),4)
	yscaling = round(1.0/(y_diff*2.8346),4)
	QUB.Report( 'x_scaling = ',round(1.0/(x_diff*2.8346),4) )
	QUB.Report( 'y_scaling = ',round(1.0/(y_diff*2.8346),4) )
	return(xscaling,yscaling)
#~ =========================================
 
##ml = open((pathToFile+TextFileName+'.txt'),'r').readlines()
#~ print info(fft)
#~ sys.exit(0)

def CopyQUBData(dataFile, segment, channel_index, into):
	dataFile.activeChannel = channel_index
	QUB.fillData(segment)
	data = segment['Channel'].data
	try:
		for i in range(len(data)):
			into[i] = float(data[i])
	except:
		pass
	del data
	segment.remove(segment['Channel'])
	

def Cube4(props):
	# get settings from QUB dialog
	Max_mV = props["Max_mV"]
	Min_mV = props["Min_mV"]
	Max_pA = props["Max_pA"]
	Min_pA = props["Min_pA"]
	Max_mmHg = props["Max_mmHg"]
	Min_mmHg = props["Min_mmHg"]
	mVperfF = props["mVperfF"]
	Min_Capacitance = props["Min_Capacitance"]
	Max_Capacitance = props["Max_Capacitance"]
	Min_InPhase = props["Min_InPhase"]
	Max_InPhase = props["Max_InPhase"]
	Min_Gain = props["Min_Gain"]
	Max_Gain = props["Max_Gain"]
	Min_Valve_1 = props["Min_Valve_1"]
	Max_Valve_1 = props["Max_Valve_1"]
	Min_Valve_2 = props["Min_Valve_2"]
	Max_Valve_2 = props["Max_Valve_2"]
	PlotWidth = props["PlotWidth"]
	PlotHeight = props["PlotHeight"]
	PlotXStart = props["PlotXStart"]
	skip = props["skip"]

	restoreDataSource = QUB.GetDataSource()
	QUB.SetDataSource('sel')                  # act on the selection
	dataFile = QUB.DataView.currData
	datasets = QUB.getChosenData()            # get the selection record as a QUB Segment
	segment = datasets['DataSet']['Segment']

	if len(segment.data) == 0:
		QUB.Report('no data selected?')
		return

	if dataFile.dataChannelCount < 9:
		QUB.Report('not enough data channels (<9)')
		return

	TextFileName = str(datasets['DataSet']['FileName'].data)
	TextFileName = os.path.splitext(TextFileName)[0]
	pathToFile, TextFileName = os.path.split(TextFileName)

	restoreActiveChannel = dataFile.activeChannel	

	NumbRows = segment.data[1] - segment.data[0] + 1
	QUB.Report('Number of Rows %d' % NumbRows)

	# build time columns from start time and sampling
	time = arange(NumbRows).astype(Float32)
	fixTime = arange(NumbRows).astype(Float32)
	startTime = segment['start'].data[0] / 1000.0      # ms -> sec
	sampling = datasets['DataSet']['sampling'].data[0] / 1000.0
	for i in range(NumbRows):
		time[i] = float(startTime + i * sampling)
		fixTime[i] = float(i * sampling)

	# copy each column from a channel of data
	volts = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 0, volts)

	amps = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 1, amps)
	for i in range(NumbRows):
		amps[i] = - amps[i]

	press = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 2, press)

	clampGain = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 3, clampGain)

	valve_1 = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 4, valve_1)
	
	valve_2 = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 5, valve_2)
	
	R = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 6, R)

	Capacitance = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 7, Capacitance)

	InPhase = arange(NumbRows).astype(Float32)
	CopyQUBData(dataFile, segment, 8, InPhase)

	QUB.SetDataSource(restoreDataSource)
	dataFile.activeChannel = restoreActiveChannel
	
	

	DecimateEvery = 1
	a = NumbRows
	#~ Estimate a good decimation number.......
	while 1:
		a = NumbRows/DecimateEvery
		if a > 1300:
			DecimateEvery += 1
		if a < 1300:
			break
	DecimateEvery = int(DecimateEvery)
	QUB.Report( 'Plotting Every '+str(DecimateEvery)+' point' )
	QUB.Report( 'Plotting '+str(int(floor(NumbRows/DecimateEvery)))+' points.' )
	#~ DecimateEvery = 10

	#~ FFT stuff
	fft_pA = fft(amps)
	numbPts = int(len(fft_pA)/2)
	fft_freq = arange(numbPts).astype(Float32)
	fft_fixed = fft_pA[0:numbPts]

	#~ Calculate R...R = (real^2 + imag^2)0.5
	fft_R = [(fft_fixed[i].real**2 + fft_fixed[i].imag**2)**0.5 for i in range(0,len(fft_fixed))] 

	i = 1
	deltaFreq = (1./numbPts)*1000
	fft_freq[0] = deltaFreq
	buf = deltaFreq
	while i <= numbPts-1:
		buf = buf + deltaFreq
		fft_freq[i] = buf
		i += 1
	QUB.Report( len(fft_freq) )
	Time_fft = [[fft_freq[i],fft_R[i]] for i in range(0,len(fft_R))]

	#~ Make Lists of Each variable, each with time in column 1
	Time_voltage = [[fixTime[i],volts[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_pA = [[fixTime[i],amps[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_pressure = [[fixTime[i],press[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_PatchClampGain = [[fixTime[i],clampGain[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_valve_1 =  [[fixTime[i],valve_1[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_valve_2 =  [[fixTime[i],valve_2[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_R =  [[fixTime[i],R[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_Capacitance =  [[fixTime[i],Capacitance[i]] for i in range(0,NumbRows,DecimateEvery)]
	Time_InPhase =  [[fixTime[i],InPhase[i]] for i in range(0,NumbRows,DecimateEvery)]

	Max_Time = floor(Time_pA[-1][0])
	#~ print 'Max Time = ',Max_Time
	#~ sys.exit(0)

	QUB.Report( Time_InPhase[0:4] )
	# [wtf] # sys.exit(0)
	#~ print 'NumbRows = ',NumbRows
	#~ print len(Time_pA)
	#~ print Time_pA[0:10]
	#~ sys.exit(0)
	c = canvas.canvas()

	epsOutputFileName = os.path.join(pathToFile, TextFileName)
	FinalTextFileName = os.path.join(pathToFile,TextFileName + '.txt')
	QUB.Report( '' )
	QUB.Report( 'Input File Name:' )
	QUB.Report( FinalTextFileName )

	voltage = c.insert(graph.graphxy(width=PlotWidth,height = PlotHeight,xpos=PlotXStart,ypos=25, x2 = None,y2=None,
		x = graph.axis.lin(title = 'time',min=0,painter=None),
		y = graph.axis.lin(title='mV',min=Min_mV,max=Max_mV)))
	voltage.plot(graph.data.list(Time_voltage, x=1, y=2), 
		[graph.style.line([style.linewidth.Thin, style.linestyle.solid])])
	QUB.Report( '' )
	QUB.Report( 'Voltage Scaling....' )
	#~ Voltage Scaling...
	voltage_xscaling,voltage_yscaling = findScaling(voltage)
	VoltageScaling = r'\tiny Voltage Scaling: '+str(voltage_yscaling)
	half_xmax,half_ymax = voltage.pos(0,Min_mV+(Max_mV-Min_mV)/2.)
	#~ print half_xmax,'  ',half_ymax,'   ',stuff
	c.text(half_xmax,half_ymax,VoltageScaling)

	min_voltage_x,min_voltage_y = voltage.pos(0,Min_mV)
	#~ print min_x,'    ',min_y
	#~ c.text(min_x,min_y,'here')

	#~ Time Scaling...
	TimeScaling = r'\tiny Time Scaling: '+str(voltage_xscaling)
	#~ xmin,ymin = voltage.pos(0,Min_mV)
	#~ print xmin,'   ',ymin,'   ',morestuff
	c.text(half_xmax,half_ymax+0.2,TimeScaling)
	c.insert(voltage)

	pA = c.insert(graph.graphxy(width=PlotWidth,height=PlotHeight,
		xpos=PlotXStart,ypos=min_voltage_y-skip, x2 = None,y2=None,
		x = graph.axis.lin(title = 'time',min=0,painter=None),
		y = graph.axis.lin(title='pA',min=Min_pA,max=Max_pA)))
	pA.plot(graph.data.list(Time_pA, x=1, y=2), 
		[graph.style.line([style.linewidth.Thin, style.linestyle.solid])])
	QUB.Report( '' )
	QUB.Report( 'Current Scaling....' )
	pA_xscaling,pA_yscaling = findScaling(pA)
	pAScaling = r'\tiny Current Scaling: '+str(pA_yscaling)
	half_xmax,half_ymax = pA.pos(0,Min_pA+(Max_pA-Min_pA)/2.)
	c.text(half_xmax,half_ymax,pAScaling)
	min_pA_x,min_pA_y = pA.pos(0,Min_pA)


	pressure = c.insert(graph.graphxy(width=PlotWidth,height=PlotHeight,
		xpos=PlotXStart,ypos=min_pA_y-skip, x2 = None,y2=None,
		x = graph.axis.lin(title = 'time',min=0,painter=None),
		y = graph.axis.lin(title='mmHg',min=Min_mmHg,max=Max_mmHg)))
	pressure.plot(graph.data.list(Time_pressure, x=1, y=2), 
		[graph.style.line([style.linewidth.Thin, style.linestyle.solid])])
	QUB.Report( '' )
	QUB.Report( 'Pressure Scaling....' )
	pressure_xscaling,pressure_yscaling = findScaling(pressure)
	PressureScaling = r'\tiny pressure Scaling: '+str(pressure_yscaling)
	half_xmax,half_ymax = pressure.pos(0,Min_mmHg+(Max_mmHg-Min_mmHg)/2.)
	c.text(half_xmax,half_ymax,PressureScaling)
	min_pressure_x,min_pressure_y = pressure.pos(0,Min_mmHg)


	capacitance = c.insert(graph.graphxy(width=PlotWidth,height=PlotHeight,
		xpos=PlotXStart,ypos=min_pressure_y-skip, x2 = None,y2=None,
		x = graph.axis.lin(title = 'time',min=0,painter=None),
		y = graph.axis.lin(title='mV (cap)',min=Min_Capacitance,max=Max_Capacitance)))
	capacitance.plot(graph.data.list(Time_Capacitance, x=1, y=2), 
		[graph.style.line([style.linewidth.Thin, style.linestyle.solid])])
	QUB.Report( '' )
	QUB.Report( 'Capacitance Scaling....' )
	capacitance_xscaling,capacitance_yscaling = findScaling(capacitance)
	CapacitanceScaling = r'\tiny Capacitance Scaling: '+str(capacitance_yscaling)
	half_xmax,half_ymax = capacitance.pos(0,Min_Capacitance+(Max_Capacitance-Min_Capacitance)/2.)
	c.text(half_xmax,half_ymax,CapacitanceScaling)
	min_Capacitance_x,min_Capacitance_y = capacitance.pos(0,Min_Capacitance)


	inphase = c.insert(graph.graphxy(width=PlotWidth,height=PlotHeight,
		xpos=PlotXStart,ypos=min_Capacitance_y-skip, x2 = None,y2=None,
		x = graph.axis.lin(title = 'time',min=0,painter=None),
		y = graph.axis.lin(title='mV (In Phase)',min=Min_InPhase,max=Max_InPhase)))
	inphase.plot(graph.data.list(Time_InPhase, x=1, y=2), 
		[graph.style.line([style.linewidth.Thin, style.linestyle.solid])])
	QUB.Report( '' )
	QUB.Report( 'In Phase Scaling....' )
	InPhase_xscaling,InPhase_yscaling = findScaling(inphase)
	InPhaseScaling = r'\tiny InPhase Scaling: '+str(InPhase_yscaling)
	half_xmax,half_ymax = inphase.pos(0,Min_InPhase+(Max_InPhase-Min_InPhase)/2.)
	c.text(half_xmax,half_ymax,InPhaseScaling)

	#~ Make a line 3 seconds long.....
	#~ print voltage.pos(0,0)[0]
	#~ print voltage.pos(0,0)[1]
	#~ print voltage.pos(3,0)[0]
	#~ print voltage.pos(3,0)[1]

	xTimeStart = inphase.pos(1,Min_InPhase)[0]
	yTimeStart = inphase.pos(1,Min_InPhase)[1]

	xTimeEnd = inphase.pos(4,Min_InPhase)[0]
	yTimeEnd = inphase.pos(4,Min_InPhase)[1]

	myTimePainter = path.line(xTimeStart,yTimeStart,xTimeEnd,yTimeEnd)
	c.stroke(myTimePainter,[style.linewidth.Thick])
	TimeLabel = r'3 sec'
	xTimeLabel = xTimeStart + (xTimeEnd  - xTimeStart)/2.
	yTimeLabel = yTimeEnd
	#~ Make a line 1 mV long
	#~ myVoltagePainter = path.line(voltage.pos(0,0)[0],voltage.pos(0,0)[1],voltage.pos(0,2)[0],voltage.pos(0,2)[1])

	#~ c.stroke(myVoltagePainter,[style.linewidth.Thick])
	c.text(xTimeLabel,yTimeLabel-0.4,TimeLabel)

	#~ print voltage.vpos(0,0)

	QUB.Report( '' )

	#~ c.insert(epsfile.epsfile(0,0,'some_eps_file'))
	c.writeEPSfile(epsOutputFileName)  # rotated =1 makes Landscape mode
	#~ pathToFile = 'D:\\Charley\\AcquisitionInput\\CHO_Cells\\Raw_CHO\\'
	epsOutputFileName = os.path.join(pathToFile, TextFileName)
	QUB.Report( '' )
	QUB.Report( 'View '+epsOutputFileName+'.eps with ghostView' )
	QUB.Report( '' )

#~ gsView = r'd:\Program Files\Ghostgum\gsview\gsview32.exe'
#~ a = os.spawnv(os.P_NOWAIT,gsView,[epsOutputFileName])
#~ print a


			#~ Patch Clamp Gain   Axopatch Telegraph Output
			#~ 0.5 mV/pA            	2.0 volts
			#~ 1                       	2.5
			#~ 2                       	3.0
			#~ 5                       	3.5
			#~ 10                      	4.0
			#~ 20                      	4.5
			#~ 50                      	5.0
			#~ 100                    	5.5
			#~ 200                    	6.0
			#~ 500                    	6.5

		#~ Best Fit:
		#~ Patch Clamp Gain = 0.0221 * exp(1.5334 * volts)

#~ 				Qub Hardware  Configuration Setup:
					#~ Inputs
	#~ I/O Channel	Input		Unit		Gain		Calibration Factor
	#~ 0			Voltage	mV		0.5		0.01 (volts/unit)
	#~ 1			Current	pA		0.5		0.1
	#~ 2			Pressure	mmHg		1.0		0.018
	#~ 3		Patch Gain		Volts		0.5		1.0
	
						#~ Outputs
	#~ I/O Channel	Output	Increment	Calibration Factor
	#~ 0			Vout		0.1		0.025 volts/unit
	#~ 1			Pout		0.1		0.009


#~ mytexter = graph.axis.texter.decimal(equalprecision=0,thousandsep=',')
#~ myXparter = graph.axis.parter.linear(tickdist=[60,6])

#~ myYparter = graph.axis.parter.linear(tickdist=[10,1])

#~ g.plot(graph.data.file("noname8xx.txt", x=1, y=3),
	#~ [graph.style.symbol(graph.style.symbol.circle,size=0.05,	
	#~ symbolattrs=[deco.filled])])




AddTaskButton( 'Cube4', # button text
               Cube4,   # function
               'Cube4',       # short text for small buttons
               0,            # show settings dialog on left-click?
               1,            # show settings dialog on right-click?
               "Ask Charles Bowman",   # tool-tip
               "Ask Charles Bowman",   # description in "Edit Buttons"
               "Cube4 properties:", # caption for the settings dialog
               [TextField("Max_mV", "Max_mV:", "50", acceptFloat),
                TextField("Min_mV", "Min_mV:", "-50", acceptFloat),
                TextField("Max_pA", "Max_pA:", "10", acceptFloat),
                TextField("Min_pA", "Min_pA:", "-10", acceptFloat),
                TextField("Max_mmHg", "Max_mmHg:", "5", acceptFloat),
                TextField("Min_mmHg", "Min_mmHg:", "-100", acceptFloat),
                TextField("mVperfF", "mVperfF:", ".0102", acceptFloat),
                TextField("Min_Capacitance", "Min_Capacitance (fF):", "-1", acceptFloat),
                TextField("Max_Capacitance", "Max_Capacitance (fF):", "1", acceptFloat),
                TextField("Min_InPhase", "Min_InPhase:", "-.2", acceptFloat),
                TextField("Max_InPhase", "Max_InPhase:", ".2", acceptFloat),
                TextField("Min_Gain", "Min_Gain:", ".1", acceptFloat),
                TextField("Max_Gain", "Max_Gain:", "1000", acceptFloat),
                TextField("Min_Valve_1", "Min_Valve_1:", "-2", acceptFloat),
                TextField("Max_Valve_1", "Max_Valve_1:", "11", acceptFloat),
                TextField("Min_Valve_2", "Min_Valve_2:", "-2", acceptFloat),
                TextField("Max_Valve_2", "Max_Valve_2:", "11", acceptFloat),
                TextField("PlotWidth", "PlotWidth (cm):", "18", acceptFloat),
                TextField("PlotHeight", "PlotHeight (cm):", "2", acceptFloat),
                TextField("PlotXStart", "PlotXStart (cm):", "2", acceptFloat),
                TextField("skip", "skip (cm):", "2.5", acceptFloat)],
               'Scripts' )
